/*************************************************
* CAST256 Header File                            *
* (C) 1999-2002 The OpenCL Project               *
*************************************************/

#ifndef OPENCL_CAST256_H__
#define OPENCL_CAST256_H__

#include <opencl/opencl.h>

namespace OpenCL {

class CAST256 : public BlockCipher
   {
   public:
      static std::string name() { return "CAST-256"; }
      static const u32bit BLOCKSIZE = 16, KEYLENGTH = 32;

      void clear() throw() { MK.clear(); RK.clear(); }
      BlockCipher* clone() const { return new CAST256; }
      CAST256() : BlockCipher("CAST-256", BLOCKSIZE, 4, 32, 4) {}
   private:
      void enc(const byte[BLOCKSIZE], byte[BLOCKSIZE]) const;
      void dec(const byte[BLOCKSIZE], byte[BLOCKSIZE]) const;
      void key(const byte[], u32bit);
      void round1(u32bit&, u32bit, u32bit, u32bit) const;
      void round2(u32bit&, u32bit, u32bit, u32bit) const;
      void round3(u32bit&, u32bit, u32bit, u32bit) const;
      static const u32bit KEY_MASK[192];
      static const byte   KEY_ROT[192];
      SecureBuffer<u32bit, 48> MK;
      SecureBuffer<byte, 48> RK;
   };

extern const u32bit CAST_SBOX1[256];
extern const u32bit CAST_SBOX2[256];
extern const u32bit CAST_SBOX3[256];
extern const u32bit CAST_SBOX4[256];

}

#endif
