/*************************************************
* ARC4 Source File                               *
* (C) 1999-2002 The OpenCL Project               *
*************************************************/

#include <opencl/arc4.h>

namespace OpenCL {

/*************************************************
* Generate ARC4 Cipher Stream                    *
*************************************************/
void ARC4::cipher(const byte in[], byte out[], u32bit length)
   {
   while(length >= buffer.size() - position)
      {
      xor_buf(out, in, buffer + position, buffer.size() - position);
      length -= (buffer.size() - position);
      in += (buffer.size() - position);
      out += (buffer.size() - position);
      generate();
      }
   xor_buf(out, in, buffer + position, length);
   position += length;
   }

/*************************************************
* Generate the internal state                    *
*************************************************/
void ARC4::generate()
   {
   for(u32bit j = 0; j != buffer.size(); j++)
      {
      X++;
      Y += state[X % 256];
      std::swap(state[X % 256], state[Y % 256]);
      buffer[j] = (byte)state[(state[X % 256] + state[Y % 256]) % 256];
      }
   position = 0;
   }

/*************************************************
* ARC4 Key Schedule                              *
*************************************************/
void ARC4::key(const byte key[], u32bit length)
   {
   clear();
   for(u32bit j = 0; j != 256; j++)
      state[j] = j;
   for(u32bit j = 0, state_index = 0; j != 256; j++)
      {
      state_index = (state_index + key[j % length] + state[j]) % 256;
      std::swap(state[j], state[state_index]);
      }
   X = Y = 0;
   for(u32bit j = 0; j <= SKIP; j += buffer.size())
      generate();
   position += (SKIP % buffer.size());
   }

/*************************************************
* Clear memory of sensitive data                 *
*************************************************/
void ARC4::clear() throw()
   {
   state.clear();
   buffer.clear();
   position = X = Y = 0;
   }

/*************************************************
* ARC4 Constructor                               *
*************************************************/
ARC4::ARC4(u32bit s) : StreamCipher(s ? "MARK4" : "ARC4", 1, 32), SKIP(s)
   {
   clear();
   }

}
