/*************************************************
* EMAC Source File                               *
* (C) 1999-2002 The OpenCL Project               *
*************************************************/

#include <opencl/emac.h>

namespace OpenCL {

/*************************************************
* Update an EMAC Calculation                     *
*************************************************/
void EMAC::add_data(const byte input[], u32bit length)
   {
   while(length)
      {
      u32bit xored = std::min(state.size() - position, length);
      xor_buf(state + position, input, xored);
      input += xored;
      length -= xored;
      position += xored;
      if(position == state.size())
         {
         f1->encrypt(state);
         position = 0;
         }
      }
   }

/*************************************************
* Finalize an EMAC Calculation                   *
*************************************************/
void EMAC::final_result(byte mac[])
   {
   for(u32bit j = position; j != state.size(); j++)
      state[j] ^= (byte)(state.size()-position);
   f1->encrypt(state);
   f2->encrypt(state, mac);
   state.clear();
   position = 0;
   }

/*************************************************
* EMAC Key Setup                                 *
*************************************************/
void EMAC::key(const byte key[], u32bit length)
   {
   if(!f1->valid_keylength(length))
      throw Invalid_Key_Length("EMAC<" + f1->name() + ">", length);

   SecureVector<byte> subkeys(2 * f1->MAXIMUM_KEYLENGTH);
   SecureVector<byte> block(f1->BLOCK_SIZE), counter(f1->BLOCK_SIZE);
   u32bit generated = 0;
   f1->set_key(key, length);
   while(generated < subkeys.size())
      {
      f1->encrypt(counter, block);
      subkeys.copy(generated, block, block.size());
      generated += block.size();
      for(s32bit j = counter.size() - 1; j >= 0; j--)
         if(++counter[j])
            break;
      }
   f1->set_key(subkeys, subkeys.size() / 2);
   f2->set_key(subkeys + subkeys.size() / 2, subkeys.size() / 2);
   }

/*************************************************
* Clear memory of sensitive data                 *
*************************************************/
void EMAC::clear() throw()
   {
   f1->clear();
   f2->clear();
   state.clear();
   position = 0;
   }

/*************************************************
* EMAC Constructor                               *
*************************************************/
EMAC::EMAC(BlockCipher* c) :
   MessageAuthenticationCode("EMAC<" + c->name() + ">", c->BLOCK_SIZE, 1, 32)
   {
   f1 = c;
   f2 = c->clone();
   state.create(OUTPUT_LENGTH);
   position = 0;
   }

}
